@@ -33,8 +33,9 @@ use Scalar::Util 'blessed';
 use constant DOCUMENT         => 'Template::Document';
 use constant EXCEPTION        => 'Template::Exception';
 use constant BADGER_EXCEPTION => 'Badger::Exception';
+use constant MSWin32          => $^O eq 'MSWin32';
 
-our $VERSION = 2.98;
+our $VERSION = '3.000';
 our $DEBUG   = 0 unless defined $DEBUG;
 our $DEBUG_FORMAT = "\n## \$file line \$line : [% \$text %] ##\n";
 our $VIEW_CLASS   = 'Template::View';
@@ -108,7 +109,7 @@ sub template {
         
         # now it's time to ask the providers, so we look to see if any 
         # prefix is specified to indicate the desired provider set.
-        if ($^O eq 'MSWin32') {
+        if (MSWin32) {
             # let C:/foo through
             $prefix = $1 if $shortname =~ s/^(\w{2,})://o;
         }
@@ -186,11 +187,16 @@ sub plugin {
     
     $self->debug("plugin($name, ", defined $args ? @$args : '[ ]', ')')
         if $self->{ DEBUG };
-    
+
+    return $self->{ _PLUGIN_CONTEXT_CACHE }{$name} if !$args && $self->{ _PLUGIN_CONTEXT_CACHE }{$name};
+
     # request the named plugin from each of the LOAD_PLUGINS providers in turn
     foreach my $provider (@{ $self->{ LOAD_PLUGINS } }) {
         ($plugin, $error) = $provider->fetch($name, $args, $self);
-        return $plugin unless $error;
+        unless($error) {
+          $self->{ _PLUGIN_CONTEXT_CACHE }{$name} = $plugin if !$args;
+          return $plugin;
+        }
         if ($error == Template::Constants::STATUS_ERROR) {
             $self->throw($plugin) if ref $plugin;
             $self->throw(Template::Constants::ERROR_PLUGIN, $plugin);
@@ -429,7 +435,7 @@ sub insert {
     FILE: foreach $file (@$files) {
         my $name = $file;
 
-        if ($^O eq 'MSWin32') {
+        if (MSWin32) {
             # let C:/foo through
             $prefix = $1 if $name =~ s/^(\w{2,})://o;
         }
@@ -761,25 +767,25 @@ sub debugging {
     my @args = @_;
 
     if (@args) {
-        if ($args[0] =~ /^(on|1)$/i) {
+        if ($args[0] eq '1' || lc($args[0]) eq 'on' ) {
             $self->{ DEBUG_DIRS } = 1;
             shift(@args);
         }
-        elsif ($args[0] =~ /^(off|0)$/i) {
+        elsif ($args[0] eq '0' || lc($args[0]) eq 'off') {
             $self->{ DEBUG_DIRS } = 0;
             shift(@args);
         }
     }
 
     if (@args) {
-        if ($args[0] =~ /^msg$/i) {
+        if (lc($args[0]) eq 'msg') {
             return unless $self->{ DEBUG_DIRS };
             my $format = $self->{ DEBUG_FORMAT };
             $format = $DEBUG_FORMAT unless defined $format;
             $format =~ s/\$(\w+)/$hash->{ $1 }/ge;
             return $format;
         }
-        elsif ($args[0] =~ /^format$/i) {
+        elsif ( lc($args[0]) eq 'format' ) {
             $self->{ DEBUG_FORMAT } = $args[1];
         }
         # else ignore
