@@ -23,7 +23,7 @@ use warnings;
 use base 'Template::Plugin';
 use CGI;
 
-our $VERSION = 2.70;
+our $VERSION = '3.000';
 
 sub new {
     my $class   = shift;
@@ -31,9 +31,9 @@ sub new {
     CGI->new(@_);
 }
 
-package CGI;
+# monkeypatch CGI::params() method to Do The Right Thing in TT land
 
-sub params {
+sub CGI::params {
     my $self = shift;
     local $" = ', ';
 
