@@ -23,7 +23,7 @@ use strict;
 use warnings;
 use base 'Template::Plugin';
 
-our $VERSION = 2.74;
+our $VERSION = '3.000';
 our $JOINT   = '&amp;';
 
 
@@ -76,6 +76,7 @@ sub args {
 sub escape {
     my $toencode = shift;
     return undef unless defined($toencode);
+    utf8::encode($toencode);
     $toencode=~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
     return $toencode;
 }
